<?php

/**
 * @file
 * Provides SDL BeGlobal Translator plugin controller.
 *
 * Check @link https://www.beglobal.com/developers/api-documentation/
 * SDL BeGlobal Cloud Translator @endlink.
 */

/**
 * SDL BeGlobal translator plugin controller.
 */
class TMGMTSDLBeGlobalTranslatorPluginController extends TMGMTDefaultTranslatorPluginController {

  const MACHINE_TRANSLATION_QUALITY = 'Q1';
  const STANDARD_QUALITY = 'Q3';
  const VERIFIED_QUALITY = 'Q4';
  const SPECIALIZED_QUALITY = 'Q5';
  const TRANSLATOR_URL = 'https://api.beglobal.com/';

  /**
   * API key used for authentication of the API calls.
   *
   * @var
   */
  public $api_key;

  /**
   * Translation service URL.
   *
   * @var string
   */
  protected $translatorUrl = self::TRANSLATOR_URL;

  /**
   * Available actions for SDL BeGlobal translator.
   *
   * @var array
   */
  protected $availableActions = array('translate', 'languages', 'detect');

  /**
   * The translation quality used for API calls.
   *
   * @var string
   */
  protected $translation_quality = self::MACHINE_TRANSLATION_QUALITY;

  /**
   * Builds an array of options containing the possible types of translation
   * quality.
   *
   * @return array
   */
  public static function getTranslationQualityOptions() {
    return array(
      self::MACHINE_TRANSLATION_QUALITY => t('Machine translation'),
      self::STANDARD_QUALITY => t('Standard translation'),
      self::VERIFIED_QUALITY => t('Verified translation'),
      self::SPECIALIZED_QUALITY => t('Professional translation'),
    );
  }

  /**
   * Translation quality getter.
   *
   * @return string
   */
  public function getTranslationQuality() {
    return $this->translation_quality;
  }

  /**
   * Translation quality setter.
   *
   * @param $translation_quality
   *
   * @return mixed
   */
  protected function setTranslationQuality($translation_quality) {
    $this->translation_quality = $translation_quality;
  }

  /**
   * Overrides TMGMTDefaultTranslatorPluginController::isAvailable().
   */
  public function isAvailable(TMGMTTranslator $translator) {
    if ($translator->getSetting('api_key')) {

      return TRUE;
    }

    return FALSE;
  }

  /**
   * Implements TMGMTTranslatorPluginControllerInterface::requestTranslation().
   */
  public function requestTranslation(TMGMTJob $job) {
    // Pull the source data array through the job and flatten it.
    $data = array_filter(tmgmt_flatten_data($job->getData()), '_tmgmt_filter_data');

    $translation = array();

    try {

      foreach ($data as $key => $value) {
        $result = $this->sdlBeGlobalRequestTranslation($job, $value['#text']);
        if (!empty($result['translation'])) {
          $translation[$key]['#text'] = $result['translation'];
        }
      }

      // The translation job has been successfully submitted.
      $job->submitted('The translation job has been submitted.');

      // Save the translated data through the job.
      // NOTE that this line of code is reached only in case all translation
      // requests succeeded.
      $job->addTranslatedData(tmgmt_unflatten_data($translation));
    }
    catch (TMGMTSDLBeGlobalException $e) {
      $job->rejected('Translation has been rejected with following error: !error',
        array('!error' => $e->getMessage()), 'error');
    }
  }

  /**
   * Helper method to do translation request.
   *
   * @param TMGMTJob $job
   * @param array|string $text
   *   Text to be translated.
   *
   * @return array
   *   Unserialized JSON containing translated texts.
   */
  protected function sdlBeGlobalRequestTranslation(TMGMTJob $job, $text) {
    return $this->doRequest($job->getTranslator(), 'translate', array(), array(
        'data' => json_encode(array(
          'from' => $this->mapToRemoteLanguage($job->getTranslator(), $job->source_language),
          'to' => $this->mapToRemoteLanguage($job->getTranslator(), $job->target_language),
          'text' => $text,
        )),
        'method' => 'POST',
        'headers' => array('Content-type' => 'application/json'),
      )
    );
  }

  /**
   * Overrides TMGMTDefaultTranslatorPluginController::getSupportedTargetLanguages().
   */
  public function getSupportedTargetLanguages(TMGMTTranslator $translator, $source_language) {
    $languages = array();

    // Request only the language codes with source language as  $source_language
    // and the quality selected by the user.
    $response = $this->doRequest($translator,'languages', array(
      'from' => $source_language,
      'quality' => &$this->translation_quality
    ));

    if (!empty($response['languageExpertise'][$this->translation_quality])) {
      foreach ($response['languageExpertise'][$this->translation_quality] as $pair) {
        $languages[$pair['languagePair']['to']['code']] = $pair['languagePair']['to']['code'];
      }
    }

    return $languages;
  }

  /**
   * Overrides TMGMTDefaultTranslatorPluginController::getSupportedLanguagePairs().
   */
  public function getSupportedLanguagePairs(TMGMTTranslator $translator) {
    $language_pairs = array();

    // Request only the language codes with the quality selected by the user.
    $response = $this->doRequest($translator, 'languages', array(
      'quality' => &$this->translation_quality
    ));

    if (!empty($response['languageExpertise'][$this->translation_quality])) {
      foreach ($response['languageExpertise'][$this->translation_quality] as $pair) {
        $language_pairs += array(
          'source_language' => $pair['languagePair']['from']['code'],
          'target_language' => $pair['languagePair']['to']['code'],
        );
      }
    }

    return $language_pairs;
  }

  /**
   * Overrides TMGMTDefaultTranslatorPluginController::getSupportedRemoteLanguages().
   */
  public function getSupportedRemoteLanguages(TMGMTTranslator $translator) {
    $languages = array();

    $response = $this->doRequest($translator,'languages', array(
      'quality' => &$this->translation_quality
    ));

    if (!empty($response['languageExpertise'][$this->translation_quality])) {
      foreach ($response['languageExpertise'][$this->translation_quality] as $pair) {
        $languages[$pair['languagePair']['to']['code']] = $pair['languagePair']['to']['code'];
      }
    }

    return $languages;
  }

  /**
   * Overrides TMGMTDefaultTranslatorPluginController::hasCheckoutSettings().
   */
  public function hasCheckoutSettings(TMGMTJob $job) {
    return FALSE;
  }

  /**
   * Implements TMGMTTranslatorPluginControllerInterface::canTranslate().
   */
  public function canTranslate(TMGMTTranslator $translator, TMGMTJob $job) {
    // If we have defined mappings for the local to remote languages we need to
    // take it into account here.
    $target_language = $this->mapToRemoteLanguage($translator, $job->target_language);
    $source_language = $this->mapToRemoteLanguage($translator, $job->source_language);

    // The job is only translatable if the translator is available and the
    // target language is among the supported ones.
    return ($this->isAvailable($translator) &&
      array_key_exists($target_language, $translator->getSupportedTargetLanguages($source_language)));
  }

  /**
   * Local method to do request to SDL BeGlobal Translate service.
   *
   * @param TMGMTTranslator $translator
   *   The translator entity to get the settings from.
   * @param string $action
   *   Action to be performed [translate, languages, detect]
   * @param array $query
   *   (Optional) Additional query params to be passed into the request.
   * @param array $options
   *   (Optional) Additional options that will be passed into drupal_http_request().
   *
   * @return array
   *   Decoded JSON response from SDL BeGlobal Translate service.
   *
   * @throws TMGMTSDLBeGlobalException
   *   - Invalid action provided
   *   - Unable to connect to SDL BeGlobal Translate service
   *   - Error returned by SDL BeGlobal Translate service
   */
  protected function doRequest(TMGMTTranslator $translator, $action, array $query = array(), array $options = array()) {
    // Prepare the request
    $this->prepareRequestInformation($translator, $action);
    $url = url($this->translatorUrl . '/' . $action, array('query' => $query));
    $options['headers']['Authorization'] = 'BeGlobal apiKey=' . $this->api_key;

    $response = drupal_http_request($url, $options);

    $this->validateResponse($response);

    // Return the processed JSON result.
    return drupal_json_decode($response->data);
  }

  /**
   * Prepare the information for the request to be send to SDL BeGlobal
   * Translate service.
   *
   * @param TMGMTTranslator $translator
   * @param string $action
   *
   * @throws TMGMTSDLBeGlobalException
   */
  protected function prepareRequestInformation(TMGMTTranslator $translator, $action){
    if (!in_array($action, $this->availableActions)) {
      throw new TMGMTSDLBeGlobalException(
        t('Invalid action requested: @action'), array('@action' => $action));
    }

    // We set the translation quality and the api key before we do the request because in the
    // constructor we do not have access to the translator.
    $this->api_key = $translator->getSetting('api_key');
    if (empty($this->api_key)) {
      throw new TMGMTSDLBeGlobalException(
        t('SDL BeGlobal Translate service requires an API key. Please configure
        your this setting in the configuration page of the translator.')
      );
    }

    $this->translation_quality = $translator->getSetting('translation_quality');
    if (empty($this->translation_quality)) {
      $this->translation_quality = self::MACHINE_TRANSLATION_QUALITY;
    }
  }

  /**
   * Validates the response received from SDL BeGlobal Translate service and
   * checks for thrown errors.
   *
   * @param object $response
   *   The response object from SDL BeGlobal Translate service.
   *
   * @throws TMGMTSDLBeGlobalException
   */
  protected function validateResponse($response) {
    if ($response->code != 200) {
      throw new TMGMTSDLBeGlobalException(
        t('Unable to connect to SDL BeGlobal Translate service due to following error: @error.'),
        array('@error' => $response->error, '@request' => $response->request)
      );
    }
    // If we do not have data or we got error.
    if (!empty($response->error) || empty($response->data)) {
      throw new TMGMTSDLBeGlobalException(
        t('SDL BeGlobal Translate service returned following error: @error'),
        array('@error' => $response->error)
      );
    }
  }

  /**
   * We provide translatorUrl setter so that we can override its value
   * in automated testing.
   *
   * @param $translator_url
   */
  final function setTranslatorURL($translator_url) {
    $this->translatorUrl = $translator_url;
  }
}
